/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.cmdline;

import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.operator.ALDOpParameterDescriptor;
import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerCmdline;
import de.unihalle.informatik.Alida.dataio.provider.ALDDataIOCmdline;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOManagerException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.helpers.ALDParser;
import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.annotations.Parameter.*;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.LinkedList;
import java.util.HashMap;

/**
 * DataIO provider for ALDOperators from command line.
 * As this provider extends {@link ALDStandardizedDataIOCmdline} it
 * implements the Alida syntax conventions.
 * Reading is done only for IN and INOUT parameters.
 * Either all OUT and INPUT parameters are written/formated or a
 * subset as specified by a format string.
 * 
 * @author posch
 *
 */
@ALDDataIOProvider
public class ALDOperatorDataIOCmdline extends ALDStandardizedDataIOCmdline {

	private boolean debug = false;

	@Override
    public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();

		classes.add( ALDOperator.class);

		return classes;
	}
	
	/** Parser for ALDOperators. Expects a comma separated list of name=value pairs enclosed in curly brackets.
     * For the class of the object to be read see {@link ALDDataIOCmdline}.
     * Each name has to be an IN or INOUT parameter name of the operator and receives its value from
     * the <code>valueString</code>.
	 * If the list of name=value pairs is empty, i.e. no parameters are to be parsed,
     * the empty string is accepted, too.
	 */
	@Override
  	public Object parse(Field field, Class<?> cl, String valueString) {
		if ( debug ) {
			System.out.println( "ALDOperatorDataIOCmdline::parse using " + valueString);
		}

		return parse( field, cl, valueString, null);
	}

	/** Parser for ALDOperators. Expects a comma separated list of name=value pairs enclosed in curly brackets.
     * For the class of the object to be read see {@link ALDDataIOCmdline}.
     * Each name has to be an IN or INOUT parameter name of the operator and receives its value from
     * the <code>valueString</code>.
	 * If the list of name=value pairs is empty, i.e. no parameters are to be parsed,
     * the empty string is accepted, too.
	 */
  	public Object parse(Field field, Class<?> cl, String valueString, ALDOperator op) {
		valueString = valueString.trim();

		String pairStr;
		if ( valueString.trim().equals("") ) {
			valueString = "{}";
		}

		if ( (pairStr = ALDParser.parseBracket( valueString, '}')) == null ) {
      		System.err.println("ALDOperatorDataIOCmdline::readData cannot find matching {} in " + 
      				valueString);

			return null;
		}

		if ( op == null ) {
			try {
				op = (ALDOperator)(cl.newInstance());
			} catch (Exception e ) {
      			System.err.println("ALDOperatorDataIOCmdline::parse cannot instantiate " + 
					cl);
				e.printStackTrace();
				return null;
			} 
		} 

		if ( ! pairStr.trim().equals("") ) {
			HashMap<String,String> nameValuePairs = ALDParser.parseNameValuePairs( pairStr.trim());
			for ( String name : nameValuePairs.keySet() )  {

				String paramValueString = nameValuePairs.get( name);
				ALDOpParameterDescriptor descr = null;
				Object value = null;
				String pName = null;
				try {
					LinkedList<String> pNames = lookupParameternames( op, name);
					pName = pNames.getFirst();
					if ( debug ) {
						System.out.println( "Use parameter <" + pName + ">" +
							" for <" + name + "> using <" + paramValueString + "> as value");
					}
					descr = op.getParameterDescriptor( pName);
				} catch (Exception e) {
					System.err.println( "ALDOperatorDataIOCmdline::parse ERROR: parametername <" + 
							name + "> not known by operator " + op.getName());
                    System.err.println( "   existing In & INOUT parameters:");
                    for ( String n : op.getInInoutNames() )
                        System.err.println( "         " + n);

					return null;
				}
				
				if ( descr.getDirection() == Direction.IN ||
					 	descr.getDirection() == Direction.INOUT ) {
						try {
							value = ALDDataIOManagerCmdline.getInstance().readData( 
									descr.getField(), descr.getMyclass(), paramValueString);
						} catch (ALDDataIOManagerException e) {
							System.err.println( "ALDOperatorDataIOCmdline::parse ERROR: reading parametername <" + 
									name + "> from <" + paramValueString + ">");
							return null;
						}
						if ( value == null ) {
							System.err.println( "ERROR: reading parameter <" + pName + "> returns null");
							System.exit(2);
						}
	
						try {
							op.setParameter( pName, value);
						} catch (ALDOperatorException e) {
							System.err.println( "ALDOperatorDataIOCmdline::parse ERROR: cannot set value of parametername <" + 
									name + ">");
							return null;
						}
	
						if ( debug ) {
							System.out.println( "parameter <" + name + "> set to " + value);
						}
					}

			}
		}
		return op;
	}
	
	/**
	 * Format parameters of this operator into a string using the formatString.
     * If <code>formatString</code> starts with a curly bracket it is assume
     * to contained a comma seprated list of name=valueString pairs enclosed in a
     * matching curly brackets.
     * In this case, only the parameters named in this list are formated where the <code>valueString</code>
     * is passed to the <code>writeData</code> of the dataIO provider handling the parameter's type.
	 * If <code>formatString</code> does not start with a curly bracket all IN and INOUT
	 * paramters are formated.
	 *
	 * @param obj	ALDOperator to be formated
     * @param formatString  
	 */
	@Override
  	public String formatAsString(Object obj, String formatString) {
		if ( formatString == null ) {
			return this.formatAsString( obj);
		} else {
            if ( formatString.charAt(0) != '{' ) {
                return formatAsString( obj);
            }

		    String pairStr = null;
            if ( (pairStr = ALDParser.parseBracket( formatString)) == null ) {
                return formatAsString( obj);
            }

			HashMap<String,String>  nameValuePairs = ALDParser.parseNameValuePairs( pairStr);
			ALDOperator op = (ALDOperator)obj;
			StringBuffer bufstr = new StringBuffer( "{ ");
			for ( String pName : nameValuePairs.keySet() ) {
				try {
					Object value = op.getParameter( pName);
					bufstr.append( pName + "=" + ALDDataIOManagerCmdline.getInstance().writeData( value, nameValuePairs.get( pName)) + " , ");
				} catch (Exception ex) {
					System.err.println( "ALDOperatorDataIOCmdline::formatAsString ERROR cannot get value for parameter <" + pName + ">");
				}
			}
			return new String( bufstr.delete( bufstr.length()-3,  bufstr.length()).append(" }"));
		}
	}

	/**
	 * Format all OUT and INOUT parameters of this operator into a string.
	 * Produces a comma separated list of name=value pairs enclosed in curly brackets.
	 *
	 * @param obj	ALDOperator to be formated
	 */
	@Override
  	public String formatAsString(Object obj) {
		StringBuffer bufstr = new StringBuffer( "{ ");

		ALDOperator op = (ALDOperator)obj;
		for ( String pName : op.getInInoutNames() ) {
			try {
				Object value = op.getParameter( pName);
				bufstr.append( pName + "=" + value.toString() + " , ");
			} catch (Exception ex) {
				System.err.println( "ALDOperatorDataIOCmdline::formatAsString ERROR cannot get value for parameter <" + pName + ">");
			}
		}

		return new String( bufstr.delete( bufstr.length()-3,  bufstr.length()).append(" }"));
	}

	/** Lookup all parameter names of the operator with prefix <code>pre</code>.
     * If one of the paramters exactly matches <code>pre</code> only this single 
     * parameter name is returned.
     * @return All parameter names with prefix <code>pre</code> or the single parameter 
     *		exactly matching <code>pre</code> 
     */
	public static LinkedList<String> lookupParameternames( ALDOperator op, String pre ) {
		LinkedList<String> names = new LinkedList<String>();

		for ( String pName : op.getParameterNames() ) {
			if ( pName.startsWith( pre) ) {
				names.add( pName);
			}
		}

		if ( names.size() > 1 ) {
		// try to find one exact match
			for ( String pName : op.getParameterNames() ) {
				if ( pName.equals( pre) ) {
					names.clear();
					names.add( pName);
					break;
				}
			}
		}

		return names;
	}
}
