/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.cmdline;

import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerCmdline;
import de.unihalle.informatik.Alida.dataio.provider.ALDDataIOCmdline;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDParametrizedClassDataIOHelper;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDParametrizedClassDummy;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOManagerException;
import de.unihalle.informatik.Alida.helpers.ALDParser;
import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.LinkedList;
import java.util.HashMap;

/**
 * DataIO provider for parametrized classes from command line.
 * As this provider extends {@link ALDStandardizedDataIOCmdline} it
 * implements the Alida syntax conventions.
 * Reading is done only for parameters annotated 
 * with {@link de.unihalle.informatik.Alida.annotations.ALDClassParameter}.
 * Either all annotated parameters are written/formated or a
 * subset as specified by a format string.
 * 
 * @author posch
 *
 */
@ALDDataIOProvider
public class ALDParametrizedClassDataIOCmdline extends ALDStandardizedDataIOCmdline {

    /**
     * Interface method to announce class for which IO is provided for
     * 
     * @return  Collection of classes provided
     */
	@Override
    public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();

		classes.add( ALDParametrizedClassDummy.class);

		return classes;
	}
	
	/** Generic parser for parametrized classes.
     * For the class of the object to be read see {@link ALDDataIOCmdline}.
	 * As a <code>valueString</code> a comma separated list of name=value pairs enclosed in curly brackets
	 * is expected. Each name has to be a member of the class or a super class which is annotated
     * with {@link de.unihalle.informatik.Alida.annotations.ALDClassParameter}.
	 * The <code>readData</code> method of the provider for the class of the member variable
     * is used to read the objects value from <code>value</code>.
	 */
	@Override
	public Object parse(Field field, Class<?> cl, String valueString) {
		String pairStr;
		if ( valueString.charAt(0) != '{' ||
				(pairStr = ALDParser.parseBracket( valueString, '}')) == null ) {
			System.err.println("ALDParametrizedClassDataIOCmdline::readData cannot find matching {} in " + 
					valueString);

			return null;
		}

		Object pClassObj = null;
		try {
			pClassObj = cl.newInstance();
		} catch (Exception e ) {
			System.err.println("ALDParametrizedClassDataIOCmdline::parse cannot instantiate " + 
					cl);
			e.printStackTrace();
			return null;
		} 

		HashMap<String,Field> fieldMap = ALDParametrizedClassDataIOHelper.getAnnotatedFields( cl);

		if ( ! pairStr.trim().equals("") ) {
			HashMap<String,String> nameValuePairs = ALDParser.parseNameValuePairs( pairStr.trim());
			for ( String name : nameValuePairs.keySet() )  {
				if ( fieldMap.containsKey( name) ) {
					Field f = fieldMap.get( name);
					Object value;
          try {
	          value = ALDDataIOManagerCmdline.getInstance().readData( 
	          															f, null, nameValuePairs.get( name));
						ALDParametrizedClassDataIOHelper.setValue( fieldMap.get( name), pClassObj, value);
          } catch (ALDDataIOManagerException e1) {
    				switch (e1.getType())
    				{
    				case NO_PROVIDER_FOUND:
    					System.err.println("[ALDParametrizedClassDataIOCmdline] Warning:" 
    						+" no provider found to parse input value... skipping!");
    					continue;
    				case UNSPECIFIED_ERROR:
    					System.err.println("[ALDParametrizedClassDataIOCmdline] Warning:"
    						+" an unspecified provider error appeared on parsing input!");
    					continue;
    				}
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				} else {
					System.err.println( "ALDParametrizedClassDataIOCmdline::parse " + cl.getName() +
							" does not contain an annotated member " + name);
					System.err.println( "   existing members:");
					for ( String n : fieldMap.keySet() ) 
						System.err.println( "         " + n);
					return null;
				}
			}
		}
		return pClassObj;
	}
	
	/**
     * Format all parameters of this parametrized class annotated
     * with {@link de.unihalle.informatik.Alida.annotations.ALDClassParameter}
	 * into a string.
     * If <code>formatString</code> starts with a curly bracket it is assume
     * to contained a comma seprated list of name=value pairs enclosed in a
     * matching curly brackets.
     * In this case, only the (annotated) members named in this list are formated where the <code>value</code>
     * is passed to the <code>writeData</code> of the dataIO provider handling the parameter's type.
	 * In extension, is a name equals <code>*</code> all members non listed in the 
	 * <code>formatString</code> are formated using the <code>value</code> of this pair.
	 *<p>
     * If <code>formatString</code> does not start with a curly bracket all annotated
     * members are formated.
	 *
	 * @param obj	parametrized class to be formated
	 * @param formatString	
	 */
  	public String formatAsString(Object obj, String formatString) {
		if ( formatString == null ) {
			return this.formatAsString( obj);
		} else {

			// if we do not find an opening bracket, format without format string
			if ( formatString.trim().charAt(0) != '{' ) {
				return formatAsString( obj);
			}
			
			String pairStr = null;
			// if we cannot parse  brackets, format without format string
			if ( (pairStr = ALDParser.parseBracket( formatString)) == null ) {
				return formatAsString( obj);
			}

			HashMap<String,String>  nameValuePairs = ALDParser.parseNameValuePairs( pairStr);
			HashMap<String,Field> fieldMap = ALDParametrizedClassDataIOHelper.getAnnotatedFields( obj.getClass());

			StringBuffer bufstr = new StringBuffer( "{ ");
			boolean written = false;
			boolean foundAsterix = false;
			String generalFormatString = null;

			for ( String name : nameValuePairs.keySet() ) {
				if ( name.equals( "*") ) {
					foundAsterix = true;
					generalFormatString = nameValuePairs.get( name);
					continue;
				}
				if ( fieldMap.containsKey( name) ) {
					if ( addParameter( name, obj, fieldMap.get( name), nameValuePairs.get( name), bufstr)  ) {
						written = true;
					}
				} else {
      				System.err.println( "ALDParametrizedClassDataIOCmdline::formatAsString " + obj.getClass().getName() +
						" does not contain an annotated member " + name);
					System.err.println( "   existing members:");
					for ( String n : fieldMap.keySet() ) 
						System.err.println( "         " + n);
				}
			}

			if ( foundAsterix ) {
				for ( String name : fieldMap.keySet() ) {
					if ( ! nameValuePairs.containsKey( name) ) {
						if ( addParameter( name, obj, fieldMap.get( name), generalFormatString, bufstr)  ) {
							written = true;
						}
					}
				}
			}

			if ( written ) {
				// we have formated at least on parameter
				return new String( bufstr.delete( bufstr.length()-3,  bufstr.length()).append(" }"));
			} else {
				return null;
			}
		}
	}

	/**
	 * Generic formater to string of parametrized classes.
	 * Output all annotated members of the class annotated
     * with {@link de.unihalle.informatik.Alida.annotations.ALDClassParameter}.
	 */
	@Override
  	public String formatAsString(Object obj) {
		StringBuffer bufstr = new StringBuffer( "{ ");

		boolean written = false;
		HashMap<String,Field> fieldMap = ALDParametrizedClassDataIOHelper.getAnnotatedFields( obj.getClass());
		for ( String name : fieldMap.keySet() ) {
			if ( addParameter( name, obj, fieldMap.get( name), "-", bufstr)  ) {
				written = true;
			}
		}

		if ( written ) {
			// we have formated at least on parameter
			return new String( bufstr.delete( bufstr.length()-3,  bufstr.length()).append(" }"));
		} else {
			return null;
		}
	}

	/** Format the parameter <code>name</code> of the object <code>obj</code> into the buffer <code>bufstr</code>
      * using <code>formatString</code> to determine formating.
	  *
	  * @param	name	parameter to be formated
	  * @param	obj	object for which to format parameter
	  * @param	field	field of parameter to be formated
	  * @param	bufstr	Strring buffer to append formated parameter
	  * @return	false if providered returns null indicating that it handled output
	  */
	private boolean addParameter( String name, Object obj, Field field, String formatString, StringBuffer bufstr) {
		boolean written = false;
		Object value = null;
		try {
			value = ALDParametrizedClassDataIOHelper.getValue( field, obj);
		} catch (Exception ex) {
			System.err.println( "ALDParametrizedClassDataIOCmdline::formatAsString ERROR cannot get value for parameter <" + name + ">");
		}

		String str;
    try {
	    str= ALDDataIOManagerCmdline.getInstance().writeData(value,formatString);
			if ( str != null ) {
				bufstr.append( name + "=" + str + " , ");
				written = true;
			} else {
				bufstr.append( name + " written using " + formatString + " , ");
			}
    } catch (ALDDataIOManagerException e) {
			switch (e.getType())
			{
			case NO_PROVIDER_FOUND:
				System.err.println("[ALDParametrizedClassDataIOCmdline] Warning: " +
						"no provider found to handle value/formatString...");
				return false;
			case UNSPECIFIED_ERROR:
				System.err.println("[ALDParametrizedClassDataIOCmdline] Warning: " +
						"an unspecified error occured on writeData() action...");
				return false;
			}
    }
		return written;
	}
}
