/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev$
 * $Date$
 * $Author$
 *
 */
package de.unihalle.informatik.Alida.demo;

import java.util.List;
import java.util.Set;

import javax.jws.soap.SOAPBinding.ParameterStyle;

import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.annotations.ALDAOperator;

/**
 * Demo operator to cluster experimental data.
 * <p>
 * First, optionally the data are normalized.
 * Second, again optionally, the data are subjected to a PCA.
 * Finally clustered via k-means.
 * 
 * @author posch
 */
@ALDAOperator(genericExecutionMode=ALDAOperator.ExecutionMode.ALL,
                level=ALDAOperator.Level.APPLICATION)
public class AnalyzeExperiment extends ALDOperator {

    /**
     * Input data
     */
    @Parameter(label= "Experimental data", required = true, 
          direction = Parameter.Direction.IN, description = "Experimental data to subject to PCA")
    private ExperimentalData experiment = null;

    @Parameter(label="Normalize data", direction = Parameter.Direction.IN, required=false,
    		description = "Initially normalize the experimental data")
    private Boolean doNormalize = true;

    @Parameter(label="use PCA", direction = Parameter.Direction.IN, required=false,
    		description = "apply PCA before clustering")
    private Boolean doPCA = true;

	/**
     * Clusters represented as a list of clusters, where each cluster
     * is a set of experiment Ids, i.e. indices into to input experimental data.
     */
    @Parameter( label= "Clusters",  
          direction = Parameter.Direction.OUT, description = "Clusters")
    private transient List<Set<Integer>> clusters= null;

    /**
     * Normalization experiment in case normalization was requested.
     */
    @Parameter(label= "Normalized data", required = true, 
          direction = Parameter.Direction.OUT, description = "Normalized data in case normalization was requested")
    private ExperimentalData normalizedExperiment = null;

    /**
     * Summary of experiment .
     */
    @Parameter(label= "Summary",
          direction = Parameter.Direction.OUT, description = "Summary of experimental data")
    private transient ExperimentalData summaryOfExperiment = null;

    /**
     * Default constructor.
     * @throws ALDOperatorException
     */
    public AnalyzeExperiment() throws ALDOperatorException {
    }

    /**
     * Constructor.
     * 
     * @param experiment    Experimental data
     * @throws ALDOperatorException
     */
    public AnalyzeExperiment(ExperimentalData _experiment) throws ALDOperatorException {
        this.experiment = _experiment;
    }

    @Override
    protected void operate()  throws ALDOperatorException,ALDProcessingDAGException {
    	if ( verbose ) {
    		System.out.println( "ALDClusterExperiment");
    		System.out.println();
    		experiment.print();
    	}
    	
    	FilterExperimentalDataOp filterOp = new FilterExperimentalDataOp( experiment);
    	filterOp.runOp();
    	experiment = filterOp.getResult();
    	
    	ALDClusterExperiment clusterOp = new ALDClusterExperiment();
    	clusterOp.setExperiment(experiment);
    	clusterOp.setDoNormalize(doNormalize);
    	clusterOp.setDoPCA(doPCA);
    	clusterOp.runOp();
    	clusters = clusterOp.getClusters();
    	normalizedExperiment = clusterOp.getNormalizedExperiment();
    	
    	SummarizeClusters summariesOp = new SummarizeClusters();
    	summariesOp.setClusters(clusters);
    	summariesOp.setExperiment(experiment);
    	summariesOp.runOp();
    	summaryOfExperiment = summariesOp.getSummaryOfExperiment();
    }
 
    /** Get value of result.
      * Explanation: Normalized experiment.
      * @return value of result
      */
    public List<Set<Integer>> getClusters(){
        return this.clusters;
    }

    /** Get value of experiment.
      * Explanation: Experimental data to be normalized.
      * @return value of data
      */
    public ExperimentalData getExperiment(){
        return this.experiment;
    }

    /** Set value of data.
      * Explanation: Experimental data to be normalized.
      * @param value New value of data
      */
    public void setExperiment( ExperimentalData value){
        this.experiment = value;
    }
}

